local Passive_HealAura = Class(function(self, player)
    self.player = player
    self.aura_triggered = false
    self.aura_can_reset = false
    self.recharge_rate = 30
    self.trigger_thresh = 0.3
    self.reset_thresh = 0.6
    self.heal_rate = 5

    self.player:ListenForEvent("healthdelta", function(inst, data) self:Reset(inst, data) end)
    self.player:ListenForEvent("attacked", function(inst, data) self:OnAttacked(inst, data) end)
    self.player:ListenForEvent("ms_clientdisconnected", function(inst, data) self:OnDisconnect(inst, data)  end)
    self.player:ListenForEvent("death", function(inst, data) self:ForceReset(inst, data) end)
end)

function Passive_HealAura:OnAttacked(inst, data)
    if not self.aura_triggered and self.player.components.health.currenthealth <= self.trigger_thresh * self.player.components.health.maxhealth then
        self.aura_triggered = true
        self.aura_can_reset = false
        self:SpawnHealingCircle()
        self.player:DoTaskInTime(self.recharge_rate, function(inst)
            inst.components.passive_healaura.aura_can_reset = true
            inst.components.passive_healaura:Reset()
    end)
    end
end

function Passive_HealAura:SpawnHealingCircle()
    local ents = FindPlayersInRange(0, 0, 0, 100, true)
    local p = self.player
    local hp = 9999
    for i, v in ipairs(ents) do
        if v.components.health.currenthealth <= hp then
            hp = v.components.health.currenthealth
            p = v
        end
    end
    local x, z, y = p.Transform:GetWorldPosition()
    local pos = Vector3(x, y, z)
    local circle = SpawnAt("healingcircle", pos)
    local cc = circle.components.heal_aura
    cc.heal_rate = self.heal_rate
    cc.caster = self.player
    circle.boosted = false
end

function Passive_HealAura:Reset(inst, data)
    if self.player.sg:HasStateTag("nomorph") or
        self.player:HasTag("playerghost") or
        self.player.components.health:IsDead() then
        return
    end

    if self.player.components.health.currenthealth >= self.reset_thresh * self.player.components.health.maxhealth then
        self:ForceReset(nil)
    end
end

function Passive_HealAura:ForceReset(inst, data)
    self.aura_triggered = false
    self.aura_can_reset = false
end

function Passive_HealAura:OnDisconnect(inst, data)
    if self.player.userid == data.userid then
        self:ForceReset()
    end
end

return Passive_HealAura
